@@ -35,10 +35,8 @@ struct _CcBackgroundChooser
   GtkBox              parent;
 
   GtkFlowBox         *flowbox;
-  GtkWidget          *popover_recent_box;
   GtkWidget          *recent_box;
   GtkFlowBox         *recent_flowbox;
-  GtkPopover         *selection_popover;
 
   gboolean            recent_selected;
 
@@ -57,8 +55,7 @@ enum
 static guint signals [N_SIGNALS];
 
 static void
-emit_background_chosen (CcBackgroundChooser        *self,
-                        CcBackgroundSelectionFlags  flags)
+emit_background_chosen (CcBackgroundChooser *self)
 {
   g_autoptr(GList) list = NULL;
   CcBackgroundItem *item;
@@ -70,9 +67,22 @@ emit_background_chosen (CcBackgroundChooser        *self,
 
   item = g_object_get_data (list->data, "item");
 
-  g_signal_emit (self, signals[BACKGROUND_CHOSEN], 0, item, flags);
+  g_signal_emit (self, signals[BACKGROUND_CHOSEN], 0, item);
+}
 
-  gtk_flow_box_unselect_all (flowbox);
+static void
+on_delete_background_clicked_cb (GtkButton *button,
+                                 BgRecentSource  *source)
+{
+  GtkWidget *parent;
+  CcBackgroundItem *item;
+
+  parent = gtk_widget_get_parent (gtk_widget_get_parent (GTK_WIDGET (button)));
+  g_assert (GTK_IS_FLOW_BOX_CHILD (parent));
+
+  item = g_object_get_data (G_OBJECT (parent), "item");
+
+  bg_recent_source_remove_item (source, item);
 }
 
 static GtkWidget*
@@ -85,6 +95,8 @@ create_widget_func (gpointer model_item,
   GtkWidget *child;
   GtkWidget *image;
   GtkWidget *icon;
+  GtkWidget *button_image;
+  GtkWidget *button = NULL;
   BgSource *source;
 
   source = BG_SOURCE (user_data);
@@ -107,9 +119,32 @@ create_widget_func (gpointer model_item,
                        NULL);
   gtk_style_context_add_class (gtk_widget_get_style_context (icon), "slideshow-emblem");
 
+
+  if (BG_IS_RECENT_SOURCE (source))
+    {
+      button_image = gtk_image_new_from_icon_name ("window-close-symbolic", GTK_ICON_SIZE_BUTTON);
+      button =  g_object_new (GTK_TYPE_BUTTON,
+                              "image", button_image,
+                              "halign", GTK_ALIGN_END,
+                              "valign", GTK_ALIGN_START,
+                              "margin", 6,
+                              "visible", TRUE,
+                              NULL);
+
+      gtk_style_context_add_class (gtk_widget_get_style_context (button), "osd");
+      gtk_style_context_add_class (gtk_widget_get_style_context (button), "remove-button");
+
+      g_signal_connect (button,
+                        "clicked",
+                        G_CALLBACK (on_delete_background_clicked_cb),
+                        source);
+    }
+
   overlay = gtk_overlay_new ();
   gtk_container_add (GTK_CONTAINER (overlay), image);
   gtk_overlay_add_overlay (GTK_OVERLAY (overlay), icon);
+  if (button)
+    gtk_overlay_add_overlay (GTK_OVERLAY (overlay), button);
   gtk_widget_show (overlay);
 
   child = g_object_new (GTK_TYPE_FLOW_BOX_CHILD,
@@ -165,55 +200,17 @@ setup_flowbox (CcBackgroundChooser *self)
                            G_CONNECT_SWAPPED);
 }
 
-static void
-on_delete_background_clicked_cb (GtkButton           *button,
-                                 CcBackgroundChooser *self)
-{
-  g_autoptr(GList) list = NULL;
-  CcBackgroundItem *item;
-
-  list = gtk_flow_box_get_selected_children (self->recent_flowbox);
-  g_assert (g_list_length (list) == 1);
-
-  item = g_object_get_data (list->data, "item");
-
-  bg_recent_source_remove_item (self->recent_source, item);
-}
-
-static void
-on_selection_desktop_lock_clicked_cb (GtkButton           *button,
-                                      CcBackgroundChooser *self)
-{
-  emit_background_chosen (self, CC_BACKGROUND_SELECTION_DESKTOP | CC_BACKGROUND_SELECTION_LOCK_SCREEN);
-  gtk_popover_popdown (self->selection_popover);
-}
-
-static void
-on_selection_desktop_clicked_cb (GtkButton           *button,
-                                 CcBackgroundChooser *self)
-{
-  emit_background_chosen (self, CC_BACKGROUND_SELECTION_DESKTOP);
-  gtk_popover_popdown (self->selection_popover);
-}
-
-static void
-on_selection_lock_clicked_cb (GtkButton           *button,
-                              CcBackgroundChooser *self)
-{
-  emit_background_chosen (self, CC_BACKGROUND_SELECTION_LOCK_SCREEN);
-  gtk_popover_popdown (self->selection_popover);
-}
-
 static void
 on_item_activated_cb (GtkFlowBox          *flowbox,
                       GtkFlowBoxChild     *child,
                       CcBackgroundChooser *self)
 {
   self->recent_selected = flowbox == self->recent_flowbox;
-  gtk_widget_set_visible (self->popover_recent_box, self->recent_selected);
-
-  gtk_popover_set_relative_to (self->selection_popover, GTK_WIDGET (child));
-  gtk_popover_popup (self->selection_popover);
+  if (self->recent_selected)
+    gtk_flow_box_unselect_all (self->flowbox);
+  else
+    gtk_flow_box_unselect_all (self->recent_flowbox);
+  emit_background_chosen (self);
 }
 
 static void
@@ -306,23 +303,16 @@ cc_background_chooser_class_init (CcBackgroundChooserClass *klass)
                                              G_SIGNAL_RUN_FIRST,
                                              0, NULL, NULL, NULL,
                                              G_TYPE_NONE,
-                                             2,
-                                             CC_TYPE_BACKGROUND_ITEM,
-                                             G_TYPE_INT);
+                                             1,
+                                             CC_TYPE_BACKGROUND_ITEM);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/background/cc-background-chooser.ui");
 
   gtk_widget_class_bind_template_child (widget_class, CcBackgroundChooser, flowbox);
-  gtk_widget_class_bind_template_child (widget_class, CcBackgroundChooser, popover_recent_box);
   gtk_widget_class_bind_template_child (widget_class, CcBackgroundChooser, recent_box);
   gtk_widget_class_bind_template_child (widget_class, CcBackgroundChooser, recent_flowbox);
-  gtk_widget_class_bind_template_child (widget_class, CcBackgroundChooser, selection_popover);
 
-  gtk_widget_class_bind_template_callback (widget_class, on_delete_background_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_item_activated_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_selection_desktop_lock_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_selection_desktop_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_selection_lock_clicked_cb);
 }
 
 static void
@@ -356,7 +346,7 @@ cc_background_chooser_select_file (CcBackgroundChooser *self)
   gtk_window_set_modal (GTK_WINDOW (filechooser), TRUE);
 
   preview = gtk_image_new ();
-  gtk_widget_set_size_request (preview, 256, -1);
+  gtk_widget_set_size_request (preview, 154, -1);
   gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (filechooser), preview);
   gtk_file_chooser_set_use_preview_label (GTK_FILE_CHOOSER (filechooser), FALSE);
   gtk_widget_show (preview);
