@@ -1026,7 +1026,7 @@ cc_color_calibrate_init (CcColorCalibrate *calibrate)
   /* load UI */
   calibrate->builder = gtk_builder_new ();
   retval = gtk_builder_add_from_resource (calibrate->builder,
-                                          "/org/gnome/control-center/color/color-calibrate.ui",
+                                          "/org/gnome/control-center/color/cc-color-calibrate.ui",
                                           &error);
   if (retval == 0)
     g_warning ("Could not load interface: %s", error->message);
