@@ -24,20 +24,47 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <arpa/inet.h>
-#include <glib-object.h>
 #include <glib/gi18n.h>
 #include <NetworkManager.h>
 
 #include "list-box-helper.h"
+#include "ce-page.h"
 #include "ce-page-ip4.h"
 #include "ui-helpers.h"
 
-#define RADIO_IS_ACTIVE(x) (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object(CE_PAGE (page)->builder, x))))
+static void ensure_empty_address_row (CEPageIP4 *self);
+static void ensure_empty_routes_row (CEPageIP4 *self);
 
-static void ensure_empty_address_row (CEPageIP4 *page);
-static void ensure_empty_routes_row (CEPageIP4 *page);
+struct _CEPageIP4
+{
+        GtkScrolledWindow parent;
 
-G_DEFINE_TYPE (CEPageIP4, ce_page_ip4, CE_TYPE_PAGE)
+        GtkBox            *address_box;
+        GtkSizeGroup      *address_sizegroup;
+        GtkSwitch         *auto_dns_switch;
+        GtkSwitch         *auto_routes_switch;
+        GtkRadioButton    *automatic_radio;
+        GtkBox            *content_box;
+        GtkRadioButton    *disabled_radio;
+        GtkEntry          *dns_entry;
+        GtkRadioButton    *local_radio;
+        GtkRadioButton    *manual_radio;
+        GtkCheckButton    *never_default_check;
+        GtkBox            *routes_box;
+        GtkSizeGroup      *routes_metric_sizegroup;
+        GtkSizeGroup      *routes_sizegroup;
+        GtkRadioButton    *shared_radio;
+
+        NMSettingIPConfig *setting;
+
+        GtkWidget      *address_list;
+        GtkWidget      *routes_list;
+};
+
+static void ce_page_iface_init (CEPageInterface *);
+
+G_DEFINE_TYPE_WITH_CODE (CEPageIP4, ce_page_ip4, GTK_TYPE_SCROLLED_WINDOW,
+                         G_IMPLEMENT_INTERFACE (ce_page_get_type (), ce_page_iface_init))
 
 enum {
         METHOD_COL_NAME,
@@ -53,42 +80,33 @@ enum {
 };
 
 static void
-method_changed (GtkToggleButton *radio, CEPageIP4 *page)
+method_changed (CEPageIP4 *self)
 {
         gboolean addr_enabled;
         gboolean dns_enabled;
         gboolean routes_enabled;
-        GtkWidget *widget;
 
-        if (RADIO_IS_ACTIVE ("radio_disabled")) {
+        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->disabled_radio)) ||
+            gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->shared_radio))) {
                 addr_enabled = FALSE;
                 dns_enabled = FALSE;
                 routes_enabled = FALSE;
         } else {
-                addr_enabled = RADIO_IS_ACTIVE ("radio_manual");
-                dns_enabled = !RADIO_IS_ACTIVE ("radio_local");
-                routes_enabled = !RADIO_IS_ACTIVE ("radio_local");
+                addr_enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->manual_radio));
+                dns_enabled = !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->local_radio));
+                routes_enabled = !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->local_radio));
         }
 
-        widget = GTK_WIDGET (gtk_builder_get_object (CE_PAGE (page)->builder, "address_section"));
-        gtk_widget_set_visible (widget, addr_enabled);
-        gtk_widget_set_sensitive (page->dns_entry, dns_enabled);
-        gtk_widget_set_sensitive (page->routes_list, routes_enabled);
-        gtk_widget_set_sensitive (page->never_default, routes_enabled);
+        gtk_widget_set_visible (GTK_WIDGET (self->address_box), addr_enabled);
+        gtk_widget_set_sensitive (GTK_WIDGET (self->dns_entry), dns_enabled);
+        gtk_widget_set_sensitive (GTK_WIDGET (self->routes_list), routes_enabled);
+        gtk_widget_set_sensitive (GTK_WIDGET (self->never_default_check), routes_enabled);
 
-        ce_page_changed (CE_PAGE (page));
+        ce_page_changed (CE_PAGE (self));
 }
 
 static void
-switch_toggled (GObject    *object,
-                GParamSpec *pspec,
-                CEPage     *page)
-{
-        ce_page_changed (page);
-}
-
-static void
-update_row_sensitivity (CEPageIP4 *page, GtkWidget *list)
+update_row_sensitivity (CEPageIP4 *self, GtkWidget *list)
 {
         GList *children, *l;
         gint rows = 0, i = 0;
@@ -114,12 +132,12 @@ update_row_sensitivity (CEPageIP4 *page, GtkWidget *list)
 }
 
 static void
-update_row_gateway_sensitivity (CEPageIP4 *page)
+update_row_gateway_sensitivity (CEPageIP4 *self)
 {
         GList *children, *l;
         gint rows = 0;
 
-        children = gtk_container_get_children (GTK_CONTAINER (page->address_list));
+        children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
         for (l = children; l; l = l->next) {
                 GtkWidget *row = l->data;
                 GtkWidget *entry;
@@ -134,23 +152,23 @@ update_row_gateway_sensitivity (CEPageIP4 *page)
 }
 
 static void
-remove_row (GtkButton *button, CEPageIP4 *page)
+remove_row (CEPageIP4 *self)
 {
         GtkWidget *list;
         GtkWidget *row;
         GtkWidget *row_box;
 
-        row_box = gtk_widget_get_parent (GTK_WIDGET (button));
+        row_box = gtk_widget_get_parent (GTK_WIDGET (self));
         row = gtk_widget_get_parent (row_box);
         list = gtk_widget_get_parent (row);
 
         gtk_container_remove (GTK_CONTAINER (list), row);
 
-        ce_page_changed (CE_PAGE (page));
+        ce_page_changed (CE_PAGE (self));
 
-        update_row_sensitivity (page, list);
-        if (list == page->address_list)
-                update_row_gateway_sensitivity (page);
+        update_row_sensitivity (self, list);
+        if (list == self->address_list)
+                update_row_gateway_sensitivity (self);
 }
 
 static gboolean
@@ -199,12 +217,11 @@ sort_first_last (gconstpointer a, gconstpointer b, gpointer data)
 }
 
 static void
-add_address_row (CEPageIP4   *page,
+add_address_row (CEPageIP4   *self,
                  const gchar *address,
                  const gchar *network,
                  const gchar *gateway)
 {
-        GtkSizeGroup *group;
         GtkWidget *row;
         GtkWidget *row_box;
         GtkWidget *widget;
@@ -217,8 +234,8 @@ add_address_row (CEPageIP4   *page,
         gtk_style_context_add_class (gtk_widget_get_style_context (row_box), "linked");
 
         widget = gtk_entry_new ();
-        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
-        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_address_row), page);
+        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), self);
+        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_address_row), self);
         g_object_set_data (G_OBJECT (row), "address", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), address);
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
@@ -226,8 +243,8 @@ add_address_row (CEPageIP4   *page,
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
         widget = gtk_entry_new ();
-        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
-        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_address_row), page);
+        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), self);
+        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_address_row), self);
         g_object_set_data (G_OBJECT (row), "network", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), network);
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
@@ -235,8 +252,8 @@ add_address_row (CEPageIP4   *page,
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
         widget = gtk_entry_new ();
-        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
-        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_address_row), page);
+        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), self);
+        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_address_row), self);
         g_object_set_data (G_OBJECT (row), "gateway", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), gateway ? gateway : "");
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
@@ -246,30 +263,29 @@ add_address_row (CEPageIP4   *page,
         delete_button = gtk_button_new ();
         gtk_widget_set_sensitive (delete_button, FALSE);
         gtk_style_context_add_class (gtk_widget_get_style_context (delete_button), "image-button");
-        g_signal_connect (delete_button, "clicked", G_CALLBACK (remove_row), page);
+        g_signal_connect_swapped (delete_button, "clicked", G_CALLBACK (remove_row), self);
         image = gtk_image_new_from_icon_name ("edit-delete-symbolic", GTK_ICON_SIZE_MENU);
         atk_object_set_name (gtk_widget_get_accessible (delete_button), _("Delete Address"));
         gtk_button_set_image (GTK_BUTTON (delete_button), image);
         gtk_container_add (GTK_CONTAINER (row_box), delete_button);
         g_object_set_data (G_OBJECT (row), "delete-button", delete_button);
 
-        group = GTK_SIZE_GROUP (gtk_builder_get_object (CE_PAGE (page)->builder, "address_sizegroup"));
-        gtk_size_group_add_widget (group, delete_button);
+        gtk_size_group_add_widget (self->address_sizegroup, delete_button);
 
         gtk_container_add (GTK_CONTAINER (row), row_box);
         gtk_widget_show_all (row);
-        gtk_container_add (GTK_CONTAINER (page->address_list), row);
+        gtk_container_add (GTK_CONTAINER (self->address_list), row);
 
-        update_row_gateway_sensitivity (page);
-        update_row_sensitivity (page, page->address_list);
+        update_row_gateway_sensitivity (self);
+        update_row_sensitivity (self, self->address_list);
 }
 
 static void
-ensure_empty_address_row (CEPageIP4 *page)
+ensure_empty_address_row (CEPageIP4 *self)
 {
         GList *children, *l;
 
-        children = gtk_container_get_children (GTK_CONTAINER (page->address_list));
+        children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
         l = children;
 
         while (l && l->next)
@@ -277,68 +293,61 @@ ensure_empty_address_row (CEPageIP4 *page)
 
         /* Add the last, stub row if needed*/
         if (!l || validate_row (l->data))
-                add_address_row (page, "", "", "");
+                add_address_row (self, "", "", "");
 
         g_list_free (children);
 }
 
 static void
-add_address_section (CEPageIP4 *page)
+add_address_box (CEPageIP4 *self)
 {
-        GtkWidget *widget;
         GtkWidget *list;
         gint i;
 
-        widget = GTK_WIDGET (gtk_builder_get_object (CE_PAGE (page)->builder, "address_section"));
-
-        page->address_list = list = gtk_list_box_new ();
+        self->address_list = list = gtk_list_box_new ();
         gtk_list_box_set_selection_mode (GTK_LIST_BOX (list), GTK_SELECTION_NONE);
         gtk_list_box_set_header_func (GTK_LIST_BOX (list), cc_list_box_update_header_func, NULL, NULL);
         gtk_list_box_set_sort_func (GTK_LIST_BOX (list), (GtkListBoxSortFunc)sort_first_last, NULL, NULL);
-        gtk_container_add (GTK_CONTAINER (widget), list);
+        gtk_container_add (GTK_CONTAINER (self->address_box), list);
 
-        for (i = 0; i < nm_setting_ip_config_get_num_addresses (page->setting); i++) {
+        for (i = 0; i < nm_setting_ip_config_get_num_addresses (self->setting); i++) {
                 NMIPAddress *addr;
                 struct in_addr tmp_addr;
                 gchar network[INET_ADDRSTRLEN + 1];
 
-                addr = nm_setting_ip_config_get_address (page->setting, i);
+                addr = nm_setting_ip_config_get_address (self->setting, i);
                 if (!addr)
                         continue;
 
                 tmp_addr.s_addr = nm_utils_ip4_prefix_to_netmask (nm_ip_address_get_prefix (addr));
                 (void) inet_ntop (AF_INET, &tmp_addr, &network[0], sizeof (network));
 
-                add_address_row (page,
+                add_address_row (self,
                                  nm_ip_address_get_address (addr),
                                  network,
-                                 i == 0 ? nm_setting_ip_config_get_gateway (page->setting) : "");
+                                 i == 0 ? nm_setting_ip_config_get_gateway (self->setting) : "");
         }
-        if (nm_setting_ip_config_get_num_addresses (page->setting) == 0)
-                ensure_empty_address_row (page);
+        if (nm_setting_ip_config_get_num_addresses (self->setting) == 0)
+                ensure_empty_address_row (self);
 
-        gtk_widget_show_all (widget);
+        gtk_widget_show_all (GTK_WIDGET (self->address_box));
 }
 
 static void
-add_dns_section (CEPageIP4 *page)
+add_dns_section (CEPageIP4 *self)
 {
-        GtkEntry *entry;
         GString *string;
         gint i;
 
-        page->auto_dns = GTK_SWITCH (gtk_builder_get_object (CE_PAGE (page)->builder, "auto_dns_switch"));
-        gtk_switch_set_active (page->auto_dns, !nm_setting_ip_config_get_ignore_auto_dns (page->setting));
-        g_signal_connect (page->auto_dns, "notify::active", G_CALLBACK (switch_toggled), page);
+        gtk_switch_set_active (self->auto_dns_switch, !nm_setting_ip_config_get_ignore_auto_dns (self->setting));
+        g_signal_connect_swapped (self->auto_dns_switch, "notify::active", G_CALLBACK (ce_page_changed), self);
 
-        page->dns_entry = GTK_WIDGET (gtk_builder_get_object (CE_PAGE (page)->builder, "dns_entry"));
-        entry = GTK_ENTRY (page->dns_entry);
         string = g_string_new ("");
 
-        for (i = 0; i < nm_setting_ip_config_get_num_dns (page->setting); i++) {
+        for (i = 0; i < nm_setting_ip_config_get_num_dns (self->setting); i++) {
                 const char *address;
 
-                address = nm_setting_ip_config_get_dns (page->setting, i);
+                address = nm_setting_ip_config_get_dns (self->setting, i);
 
                 if (i > 0)
                         g_string_append (string, ", ");
@@ -346,21 +355,20 @@ add_dns_section (CEPageIP4 *page)
                 g_string_append (string, address);
         }
 
-        gtk_entry_set_text (entry, string->str);
+        gtk_entry_set_text (self->dns_entry, string->str);
 
-        g_signal_connect_swapped (entry, "notify::text", G_CALLBACK (ce_page_changed), page);
+        g_signal_connect_swapped (self->dns_entry, "notify::text", G_CALLBACK (ce_page_changed), self);
 
         g_string_free (string, TRUE);
 }
 
 static void
-add_route_row (CEPageIP4   *page,
+add_route_row (CEPageIP4   *self,
                const gchar *address,
                const gchar *netmask,
                const gchar *gateway,
                gint         metric)
 {
-        GtkSizeGroup *group;
         GtkWidget *row;
         GtkWidget *row_box;
         GtkWidget *widget;
@@ -373,8 +381,8 @@ add_route_row (CEPageIP4   *page,
         gtk_style_context_add_class (gtk_widget_get_style_context (row_box), "linked");
 
         widget = gtk_entry_new ();
-        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
-        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_routes_row), page);
+        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), self);
+        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self);
         g_object_set_data (G_OBJECT (row), "address", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), address);
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
@@ -382,8 +390,8 @@ add_route_row (CEPageIP4   *page,
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
         widget = gtk_entry_new ();
-        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
-        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_routes_row), page);
+        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), self);
+        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self);
         g_object_set_data (G_OBJECT (row), "netmask", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), netmask);
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
@@ -391,8 +399,8 @@ add_route_row (CEPageIP4   *page,
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
         widget = gtk_entry_new ();
-        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
-        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_routes_row), page);
+        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), self);
+        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self);
         g_object_set_data (G_OBJECT (row), "gateway", widget);
         gtk_entry_set_text (GTK_ENTRY (widget), gateway ? gateway : "");
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
@@ -400,24 +408,22 @@ add_route_row (CEPageIP4   *page,
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
         widget = gtk_entry_new ();
-        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), page);
-        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_routes_row), page);
+        g_signal_connect_swapped (widget, "changed", G_CALLBACK (ce_page_changed), self);
+        g_signal_connect_swapped (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self);
         g_object_set_data (G_OBJECT (row), "metric", widget);
         if (metric >= 0) {
-                gchar *s = g_strdup_printf ("%d", metric);
+                g_autofree gchar *s = g_strdup_printf ("%d", metric);
                 gtk_entry_set_text (GTK_ENTRY (widget), s);
-                g_free (s);
         }
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 5);
         gtk_widget_set_hexpand (widget, TRUE);
         gtk_container_add (GTK_CONTAINER (row_box), widget);
 
-        group = GTK_SIZE_GROUP (gtk_builder_get_object (CE_PAGE (page)->builder, "routes_metric_sizegroup"));
-        gtk_size_group_add_widget (group, widget);
+        gtk_size_group_add_widget (self->routes_metric_sizegroup, widget);
 
         delete_button = gtk_button_new ();
         gtk_style_context_add_class (gtk_widget_get_style_context (delete_button), "image-button");
-        g_signal_connect (delete_button, "clicked", G_CALLBACK (remove_row), page);
+        g_signal_connect_swapped (delete_button, "clicked", G_CALLBACK (remove_row), self);
         image = gtk_image_new_from_icon_name ("edit-delete-symbolic", GTK_ICON_SIZE_MENU);
         atk_object_set_name (gtk_widget_get_accessible (delete_button), _("Delete Route"));
         gtk_button_set_image (GTK_BUTTON (delete_button), image);
@@ -426,22 +432,21 @@ add_route_row (CEPageIP4   *page,
         gtk_container_add (GTK_CONTAINER (row_box), delete_button);
         g_object_set_data (G_OBJECT (row), "delete-button", delete_button);
 
-        group = GTK_SIZE_GROUP (gtk_builder_get_object (CE_PAGE (page)->builder, "routes_sizegroup"));
-        gtk_size_group_add_widget (group, delete_button);
+        gtk_size_group_add_widget (self->routes_sizegroup, delete_button);
 
         gtk_container_add (GTK_CONTAINER (row), row_box);
         gtk_widget_show_all (row);
-        gtk_container_add (GTK_CONTAINER (page->routes_list), row);
+        gtk_container_add (GTK_CONTAINER (self->routes_list), row);
 
-        update_row_sensitivity (page, page->routes_list);
+        update_row_sensitivity (self, self->routes_list);
 }
 
 static void
-ensure_empty_routes_row (CEPageIP4 *page)
+ensure_empty_routes_row (CEPageIP4 *self)
 {
         GList *children, *l;
 
-        children = gtk_container_get_children (GTK_CONTAINER (page->routes_list));
+        children = gtk_container_get_children (GTK_CONTAINER (self->routes_list));
         l = children;
 
         while (l && l->next)
@@ -449,85 +454,69 @@ ensure_empty_routes_row (CEPageIP4 *page)
 
         /* Add the last, stub row if needed*/
         if (!l || validate_row (l->data))
-                add_route_row (page, "", "", "", -1);
+                add_route_row (self, "", "", "", -1);
 
         g_list_free (children);
 }
 
 static void
-add_routes_section (CEPageIP4 *page)
+add_routes_box (CEPageIP4 *self)
 {
-        GtkWidget *widget;
         GtkWidget *list;
         gint i;
 
-        widget = GTK_WIDGET (gtk_builder_get_object (CE_PAGE (page)->builder, "routes_section"));
-
-        page->routes_list = list = gtk_list_box_new ();
+        self->routes_list = list = gtk_list_box_new ();
         gtk_list_box_set_selection_mode (GTK_LIST_BOX (list), GTK_SELECTION_NONE);
         gtk_list_box_set_header_func (GTK_LIST_BOX (list), cc_list_box_update_header_func, NULL, NULL);
         gtk_list_box_set_sort_func (GTK_LIST_BOX (list), (GtkListBoxSortFunc)sort_first_last, NULL, NULL);
-        gtk_container_add (GTK_CONTAINER (widget), list);
-        page->auto_routes = GTK_SWITCH (gtk_builder_get_object (CE_PAGE (page)->builder, "auto_routes_switch"));
-        gtk_switch_set_active (page->auto_routes, !nm_setting_ip_config_get_ignore_auto_routes (page->setting));
-        g_signal_connect (page->auto_routes, "notify::active", G_CALLBACK (switch_toggled), page);
+        gtk_container_add (GTK_CONTAINER (self->routes_box), list);
+        gtk_switch_set_active (self->auto_routes_switch, !nm_setting_ip_config_get_ignore_auto_routes (self->setting));
+        g_signal_connect_swapped (self->auto_routes_switch, "notify::active", G_CALLBACK (ce_page_changed), self);
 
 
-        for (i = 0; i < nm_setting_ip_config_get_num_routes (page->setting); i++) {
+        for (i = 0; i < nm_setting_ip_config_get_num_routes (self->setting); i++) {
                 NMIPRoute *route;
                 struct in_addr tmp_addr;
                 gchar netmask[INET_ADDRSTRLEN + 1];
 
-                route = nm_setting_ip_config_get_route (page->setting, i);
+                route = nm_setting_ip_config_get_route (self->setting, i);
                 if (!route)
                         continue;
 
                 tmp_addr.s_addr = nm_utils_ip4_prefix_to_netmask (nm_ip_route_get_prefix (route));
                 (void) inet_ntop (AF_INET, &tmp_addr, &netmask[0], sizeof (netmask));
 
-                add_route_row (page,
+                add_route_row (self,
                                nm_ip_route_get_dest (route),
                                netmask,
                                nm_ip_route_get_next_hop (route),
                                nm_ip_route_get_metric (route));
         }
-        if (nm_setting_ip_config_get_num_routes (page->setting) == 0)
-                ensure_empty_routes_row (page);
+        if (nm_setting_ip_config_get_num_routes (self->setting) == 0)
+                ensure_empty_routes_row (self);
 
-        gtk_widget_show_all (widget);
+        gtk_widget_show_all (GTK_WIDGET (self->routes_box));
 }
 
-enum
-{
-        RADIO_AUTOMATIC,
-        RADIO_LOCAL,
-        RADIO_MANUAL,
-        RADIO_DISABLED,
-        N_RADIO
-};
-
 static void
-connect_ip4_page (CEPageIP4 *page)
+connect_ip4_page (CEPageIP4 *self)
 {
-        GtkToggleButton *radios[N_RADIO];
-        GtkWidget *content;
         const gchar *str_method;
-        gboolean disabled;
-        guint method, i;
+        guint method;
 
-        add_address_section (page);
-        add_dns_section (page);
-        add_routes_section (page);
+        add_address_box (self);
+        add_dns_section (self);
+        add_routes_box (self);
 
-        page->disabled = GTK_TOGGLE_BUTTON (gtk_builder_get_object (CE_PAGE (page)->builder, "radio_disabled"));
+        str_method = nm_setting_ip_config_get_method (self->setting);
+        g_signal_connect_swapped (self->disabled_radio, "notify::active", G_CALLBACK (ce_page_changed), self);
+        g_object_bind_property (self->disabled_radio, "active",
+                                self->content_box, "sensitive",
+                                G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
 
-        str_method = nm_setting_ip_config_get_method (page->setting);
-        disabled = g_strcmp0 (str_method, NM_SETTING_IP4_CONFIG_METHOD_DISABLED) == 0;
-        gtk_toggle_button_set_active (page->disabled, disabled);
-        g_signal_connect_swapped (page->disabled, "notify::active", G_CALLBACK (ce_page_changed), page);
-        content = GTK_WIDGET (gtk_builder_get_object (CE_PAGE (page)->builder, "page_content"));
-        g_object_bind_property (page->disabled, "active",
-                                content, "sensitive",
+        g_signal_connect_swapped (self->shared_radio, "notify::active", G_CALLBACK (ce_page_changed), self);
+        g_object_bind_property (self->shared_radio, "active",
+                                self->content_box, "sensitive",
                                 G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
 
         method = IP4_METHOD_AUTO;
@@ -541,38 +530,36 @@ connect_ip4_page (CEPageIP4 *page)
                 method = IP4_METHOD_DISABLED;
         }
 
-        page->never_default = GTK_WIDGET (gtk_builder_get_object (CE_PAGE (page)->builder, "never_default_check"));
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->never_default),
-                                      nm_setting_ip_config_get_never_default (page->setting));
-        g_signal_connect_swapped (page->never_default, "toggled", G_CALLBACK (ce_page_changed), page);
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->never_default_check),
+                                      nm_setting_ip_config_get_never_default (self->setting));
+        g_signal_connect_swapped (self->never_default_check, "toggled", G_CALLBACK (ce_page_changed), self);
 
-        /* Connect radio buttons */
-        radios[RADIO_AUTOMATIC] = GTK_TOGGLE_BUTTON (gtk_builder_get_object (CE_PAGE (page)->builder, "radio_automatic"));
-        radios[RADIO_LOCAL] = GTK_TOGGLE_BUTTON (gtk_builder_get_object (CE_PAGE (page)->builder, "radio_local"));
-        radios[RADIO_MANUAL] = GTK_TOGGLE_BUTTON (gtk_builder_get_object (CE_PAGE (page)->builder, "radio_manual"));
-        radios[RADIO_DISABLED] = page->disabled;
-
-        for (i = RADIO_AUTOMATIC; i < RADIO_DISABLED; i++)
-                g_signal_connect (radios[i], "toggled", G_CALLBACK (method_changed), page);
+        g_signal_connect_swapped (self->automatic_radio, "toggled", G_CALLBACK (method_changed), self);
+        g_signal_connect_swapped (self->local_radio, "toggled", G_CALLBACK (method_changed), self);
+        g_signal_connect_swapped (self->manual_radio, "toggled", G_CALLBACK (method_changed), self);
+        g_signal_connect_swapped (self->disabled_radio, "toggled", G_CALLBACK (method_changed), self);
 
         switch (method) {
         case IP4_METHOD_AUTO:
-                gtk_toggle_button_set_active (radios[RADIO_AUTOMATIC], TRUE);
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->automatic_radio), TRUE);
                 break;
         case IP4_METHOD_LINK_LOCAL:
-                gtk_toggle_button_set_active (radios[RADIO_LOCAL], TRUE);
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->local_radio), TRUE);
                 break;
         case IP4_METHOD_MANUAL:
-                gtk_toggle_button_set_active (radios[RADIO_MANUAL], TRUE);
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->manual_radio), TRUE);
+                break;
+        case IP4_METHOD_SHARED:
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->shared_radio), TRUE);
                 break;
         case IP4_METHOD_DISABLED:
-                gtk_toggle_button_set_active (radios[RADIO_DISABLED], TRUE);
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->disabled_radio), TRUE);
                 break;
         default:
                 break;
         }
 
-        method_changed (NULL, page);
+        method_changed (self);
 }
 
 static gboolean
@@ -602,7 +589,7 @@ parse_netmask (const char *str, guint32 *prefix)
 }
 
 static gboolean
-ui_to_setting (CEPageIP4 *page)
+ui_to_setting (CEPageIP4 *self)
 {
         const gchar *method;
         gboolean ignore_auto_dns;
@@ -618,20 +605,20 @@ ui_to_setting (CEPageIP4 *page)
         gchar *dns_text = NULL;
         guint i;
 
-        if (gtk_toggle_button_get_active (page->disabled)) {
+        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->disabled_radio)))
                 method = NM_SETTING_IP4_CONFIG_METHOD_DISABLED;
-        } else {
-                if (RADIO_IS_ACTIVE ("radio_automatic"))
-                        method = NM_SETTING_IP4_CONFIG_METHOD_AUTO;
-                else if (RADIO_IS_ACTIVE ("radio_local"))
-                        method = NM_SETTING_IP4_CONFIG_METHOD_LINK_LOCAL;
-                else if (RADIO_IS_ACTIVE ("radio_manual"))
-                        method = NM_SETTING_IP4_CONFIG_METHOD_MANUAL;
-        }
+        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->automatic_radio)))
+                method = NM_SETTING_IP4_CONFIG_METHOD_AUTO;
+        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->local_radio)))
+                method = NM_SETTING_IP4_CONFIG_METHOD_LINK_LOCAL;
+        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->manual_radio)))
+                method = NM_SETTING_IP4_CONFIG_METHOD_MANUAL;
+        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->shared_radio)))
+                method = NM_SETTING_IP4_CONFIG_METHOD_SHARED;
 
         addresses = g_ptr_array_new_with_free_func ((GDestroyNotify) nm_ip_address_unref);
         if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
-                children = gtk_container_get_children (GTK_CONTAINER (page->address_list));
+                children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
         else
                 children = NULL;
 
@@ -698,7 +685,7 @@ ui_to_setting (CEPageIP4 *page)
                         g_ptr_array_add (addresses, addr);
 
                 if (!l || !l->next)
-                        ensure_empty_address_row (page);
+                        ensure_empty_address_row (self);
         }
         g_list_free (children);
 
@@ -708,7 +695,7 @@ ui_to_setting (CEPageIP4 *page)
         }
 
         dns_servers = g_ptr_array_new_with_free_func (g_free);
-        dns_text = g_strstrip (g_strdup (gtk_entry_get_text (GTK_ENTRY (page->dns_entry))));
+        dns_text = g_strstrip (g_strdup (gtk_entry_get_text (GTK_ENTRY (self->dns_entry))));
         if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO) ||
             g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
                 dns_addresses = g_strsplit_set (dns_text, ", ", -1);
@@ -725,11 +712,11 @@ ui_to_setting (CEPageIP4 *page)
 
                 if (!nm_utils_ipaddr_valid (AF_INET, text)) {
                         g_ptr_array_remove_range (dns_servers, 0, dns_servers->len);
-                        widget_set_error (page->dns_entry);
+                        widget_set_error (GTK_WIDGET (self->dns_entry));
                         ret = FALSE;
                         break;
                 } else {
-                        widget_unset_error (page->dns_entry);
+                        widget_unset_error (GTK_WIDGET (self->dns_entry));
                         g_ptr_array_add (dns_servers, g_strdup (text));
                 }
         }
@@ -745,7 +732,7 @@ ui_to_setting (CEPageIP4 *page)
         routes = g_ptr_array_new_with_free_func ((GDestroyNotify) nm_ip_route_unref);
         if (g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO) ||
             g_str_equal (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
-                children = gtk_container_get_children (GTK_CONTAINER (page->routes_list));
+                children = gtk_container_get_children (GTK_CONTAINER (self->routes_list));
         else
                 children = NULL;
 
@@ -817,7 +804,7 @@ ui_to_setting (CEPageIP4 *page)
                         g_ptr_array_add (routes, route);
 
                 if (!l || !l->next)
-                        ensure_empty_routes_row (page);
+                        ensure_empty_routes_row (self);
         }
         g_list_free (children);
 
@@ -829,11 +816,11 @@ ui_to_setting (CEPageIP4 *page)
         if (!ret)
                 goto out;
 
-        ignore_auto_dns = !gtk_switch_get_active (page->auto_dns);
-        ignore_auto_routes = !gtk_switch_get_active (page->auto_routes);
-        never_default = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->never_default));
+        ignore_auto_dns = !gtk_switch_get_active (self->auto_dns_switch);
+        ignore_auto_routes = !gtk_switch_get_active (self->auto_routes_switch);
+        never_default = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->never_default_check));
 
-        g_object_set (page->setting,
+        g_object_set (self->setting,
                       NM_SETTING_IP_CONFIG_METHOD, method,
                       NM_SETTING_IP_CONFIG_ADDRESSES, addresses,
                       NM_SETTING_IP_CONFIG_GATEWAY, default_gateway,
@@ -859,49 +846,75 @@ out:
         return ret;
 }
 
-static gboolean
-validate (CEPage        *page,
-          NMConnection  *connection,
-          GError       **error)
+static const gchar *
+ce_page_ip4_get_title (CEPage *page)
 {
-        if (!ui_to_setting (CE_PAGE_IP4 (page)))
+        return _("IPv4");
+}
+
+static gboolean
+ce_page_ip4_validate (CEPage        *self,
+                      NMConnection  *connection,
+                      GError       **error)
+{
+        if (!ui_to_setting (CE_PAGE_IP4 (self)))
                 return FALSE;
 
-        return nm_setting_verify (NM_SETTING (CE_PAGE_IP4 (page)->setting), NULL, error);
+        return nm_setting_verify (NM_SETTING (CE_PAGE_IP4 (self)->setting), NULL, error);
 }
 
 static void
-ce_page_ip4_init (CEPageIP4 *page)
+ce_page_ip4_init (CEPageIP4 *self)
 {
+        gtk_widget_init_template (GTK_WIDGET (self));
 }
 
 static void
-ce_page_ip4_class_init (CEPageIP4Class *class)
+ce_page_ip4_class_init (CEPageIP4Class *klass)
 {
-        CEPageClass *page_class= CE_PAGE_CLASS (class);
+        GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-        page_class->validate = validate;
+        gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/network/ip4-page.ui");
+
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, address_box);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, address_sizegroup);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, auto_dns_switch);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, auto_routes_switch);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, automatic_radio);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, content_box);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, disabled_radio);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, dns_entry);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, local_radio);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, manual_radio);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, never_default_check);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, routes_box);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, routes_metric_sizegroup);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, routes_sizegroup);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP4, shared_radio);
 }
 
-CEPage *
+static void
+ce_page_iface_init (CEPageInterface *iface)
+{
+        iface->get_title = ce_page_ip4_get_title;
+        iface->validate = ce_page_ip4_validate;
+}
+
+CEPageIP4 *
 ce_page_ip4_new (NMConnection     *connection,
                  NMClient         *client)
 {
-        CEPageIP4 *page;
+        CEPageIP4 *self;
 
-        page = CE_PAGE_IP4 (ce_page_new (CE_TYPE_PAGE_IP4,
-                                           connection,
-                                           client,
-                                           "/org/gnome/control-center/network/ip4-page.ui",
-                                           _("IPv4")));
+        self = CE_PAGE_IP4 (g_object_new (ce_page_ip4_get_type (), NULL));
 
-        page->setting = nm_connection_get_setting_ip4_config (connection);
-        if (!page->setting) {
-                page->setting = NM_SETTING_IP_CONFIG (nm_setting_ip4_config_new ());
-                nm_connection_add_setting (connection, NM_SETTING (page->setting));
+        self->setting = nm_connection_get_setting_ip4_config (connection);
+        if (!self->setting) {
+                self->setting = NM_SETTING_IP_CONFIG (nm_setting_ip4_config_new ());
+                nm_connection_add_setting (connection, NM_SETTING (self->setting));
         }
 
-        connect_ip4_page (page);
+        connect_ip4_page (self);
 
-        return CE_PAGE (page);
+        return self;
 }
