@@ -20,19 +20,16 @@
  * Copyright 2007 - 2014 Red Hat, Inc.
  */
 
-#include "nm-default.h"
-
-#include <ctype.h>
-#include <string.h>
+#include <glib/gi18n.h>
 
 #include "eap-method.h"
-#include "wireless-security.h"
+#include "eap-method-tls.h"
 #include "helpers.h"
 #include "nma-ui-utils.h"
-#include "utils.h"
+#include "ui-helpers.h"
 
 struct _EAPMethodTLS {
-	EAPMethod parent;
+	GtkGrid parent;
 
 	GtkFileChooserButton *ca_cert_button;
 	GtkLabel             *ca_cert_label;
@@ -47,10 +44,26 @@ struct _EAPMethodTLS {
 	GtkFileChooserButton *user_cert_button;
 	GtkLabel             *user_cert_label;
 
-	WirelessSecurity *sec_parent;
-	gboolean editing_connection;
+	gchar *username;
+	gchar *password;
+	gboolean show_password;
 };
 
+static void eap_method_iface_init (EAPMethodInterface *);
+
+G_DEFINE_TYPE_WITH_CODE (EAPMethodTLS, eap_method_tls, GTK_TYPE_GRID,
+                         G_IMPLEMENT_INTERFACE (eap_method_get_type (), eap_method_iface_init))
+
+static void
+eap_method_tls_dispose (GObject *object)
+{
+	EAPMethodTLS *self = EAP_METHOD_TLS (object);
+
+	g_clear_pointer (&self->username, g_free);
+	g_clear_pointer (&self->password, g_free);
+
+	G_OBJECT_CLASS (eap_method_tls_parent_class)->dispose (object);
+}
 
 static void
 show_toggled_cb (EAPMethodTLS *self)
@@ -62,9 +75,9 @@ show_toggled_cb (EAPMethodTLS *self)
 }
 
 static gboolean
-validate (EAPMethod *parent, GError **error)
+validate (EAPMethod *method, GError **error)
 {
-	EAPMethodTLS *self = (EAPMethodTLS *) parent;
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
 	NMSetting8021xCKFormat format = NM_SETTING_802_1X_CK_FORMAT_UNKNOWN;
 	const char *password, *identity;
 	g_autoptr(GError) ca_cert_error = NULL;
@@ -88,12 +101,16 @@ validate (EAPMethod *parent, GError **error)
 			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("invalid EAP-TLS CA certificate: %s"), ca_cert_error->message);
 			ret = FALSE;
 		}
-	} else if (eap_method_ca_cert_required (GTK_TOGGLE_BUTTON (self->ca_cert_not_required_check),
-	                                        GTK_FILE_CHOOSER (self->ca_cert_button))) {
-		widget_set_error (GTK_WIDGET (self->ca_cert_button));
-		if (ret) {
-			g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("invalid EAP-TLS CA certificate: no certificate specified"));
-			ret = FALSE;
+	} else if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->ca_cert_not_required_check))) {
+		g_autofree gchar *filename = NULL;
+
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (self->ca_cert_button));
+		if (filename == NULL) {
+			widget_set_error (GTK_WIDGET (self->ca_cert_button));
+			if (ret) {
+				g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("invalid EAP-TLS CA certificate: no certificate specified"));
+				ret = FALSE;
+			}
 		}
 	}
 
@@ -130,13 +147,13 @@ ca_cert_not_required_toggled (EAPMethodTLS *self)
 {
 	eap_method_ca_cert_not_required_toggled (GTK_TOGGLE_BUTTON (self->ca_cert_not_required_check),
 	                                         GTK_FILE_CHOOSER (self->ca_cert_button));
-	wireless_security_notify_changed (self->sec_parent);
+	eap_method_emit_changed (EAP_METHOD (self));
 }
 
 static void
-add_to_size_group (EAPMethod *parent, GtkSizeGroup *group)
+add_to_size_group (EAPMethod *method, GtkSizeGroup *group)
 {
-	EAPMethodTLS *self = (EAPMethodTLS *) parent;
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
 
 	gtk_size_group_add_widget (group, GTK_WIDGET (self->ca_cert_not_required_check));
 	gtk_size_group_add_widget (group, GTK_WIDGET (self->identity_label));
@@ -147,9 +164,9 @@ add_to_size_group (EAPMethod *parent, GtkSizeGroup *group)
 }
 
 static void
-fill_connection (EAPMethod *parent, NMConnection *connection, NMSettingSecretFlags flags)
+fill_connection (EAPMethod *method, NMConnection *connection, NMSettingSecretFlags flags)
 {
-	EAPMethodTLS *self = (EAPMethodTLS *) parent;
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
 	NMSetting8021xCKFormat format = NM_SETTING_802_1X_CK_FORMAT_UNKNOWN;
 	NMSetting8021x *s_8021x;
 	NMSettingSecretFlags secret_flags;
@@ -157,14 +174,13 @@ fill_connection (EAPMethod *parent, NMConnection *connection, NMSettingSecretFla
 	g_autofree gchar *pk_filename = NULL;
 	const char *password = NULL;
 	gboolean ca_cert_error = FALSE;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GError) error2 = NULL;
 
 	s_8021x = nm_connection_get_setting_802_1x (connection);
 	g_assert (s_8021x);
 
-	if (parent->phase2)
-		g_object_set (s_8021x, NM_SETTING_802_1X_PHASE2_AUTH, "tls", NULL);
-	else
-		nm_setting_802_1x_add_eap_method (s_8021x, "tls");
+	nm_setting_802_1x_add_eap_method (s_8021x, "tls");
 
 	g_object_set (s_8021x, NM_SETTING_802_1X_IDENTITY, gtk_entry_get_text (self->identity_entry), NULL);
 
@@ -174,30 +190,22 @@ fill_connection (EAPMethod *parent, NMConnection *connection, NMSettingSecretFla
 	pk_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (self->private_key_button));
 	g_assert (pk_filename);
 
-	if (parent->phase2) {
-		g_autoptr(GError) error = NULL;
-		if (!nm_setting_802_1x_set_phase2_private_key (s_8021x, pk_filename, password, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error))
-			g_warning ("Couldn't read phase2 private key '%s': %s", pk_filename, error ? error->message : "(unknown)");
-	} else {
-		g_autoptr(GError) error = NULL;
-		if (!nm_setting_802_1x_set_private_key (s_8021x, pk_filename, password, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error))
-			g_warning ("Couldn't read private key '%s': %s", pk_filename, error ? error->message : "(unknown)");
-	}
+	if (!nm_setting_802_1x_set_private_key (s_8021x, pk_filename, password, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error))
+		g_warning ("Couldn't read private key '%s': %s", pk_filename, error ? error->message : "(unknown)");
 
 	/* Save 802.1X password flags to the connection */
 	secret_flags = nma_utils_menu_to_secret_flags (GTK_WIDGET (self->private_key_password_entry));
-	nm_setting_set_secret_flags (NM_SETTING (s_8021x), parent->password_flags_name,
+	nm_setting_set_secret_flags (NM_SETTING (s_8021x), NM_SETTING_802_1X_PRIVATE_KEY_PASSWORD,
 	                             secret_flags, NULL);
 
 	/* Update secret flags and popup when editing the connection */
-	if (self->editing_connection) {
-		nma_utils_update_password_storage (GTK_WIDGET (self->private_key_password_entry), secret_flags,
-		                                   NM_SETTING (s_8021x), parent->password_flags_name);
-	}
+	nma_utils_update_password_storage (GTK_WIDGET (self->private_key_password_entry), secret_flags,
+	                                   NM_SETTING (s_8021x), NM_SETTING_802_1X_PRIVATE_KEY_PASSWORD);
 
 	/* TLS client certificate */
 	if (format != NM_SETTING_802_1X_CK_FORMAT_PKCS12) {
 		g_autofree gchar *cc_filename = NULL;
+		g_autoptr(GError) error = NULL;
 
 		/* If the key is pkcs#12 nm_setting_802_1x_set_private_key() already
 		 * set the client certificate for us.
@@ -206,41 +214,24 @@ fill_connection (EAPMethod *parent, NMConnection *connection, NMSettingSecretFla
 		g_assert (cc_filename);
 
 		format = NM_SETTING_802_1X_CK_FORMAT_UNKNOWN;
-		if (parent->phase2) {
-			g_autoptr(GError) error = NULL;
-			if (!nm_setting_802_1x_set_phase2_client_cert (s_8021x, cc_filename, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error))
-				g_warning ("Couldn't read phase2 client certificate '%s': %s", cc_filename, error ? error->message : "(unknown)");
-		} else {
-			g_autoptr(GError) error = NULL;
-			if (!nm_setting_802_1x_set_client_cert (s_8021x, cc_filename, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error))
-				g_warning ("Couldn't read client certificate '%s': %s", cc_filename, error ? error->message : "(unknown)");
-		}
+		if (!nm_setting_802_1x_set_client_cert (s_8021x, cc_filename, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error))
+			g_warning ("Couldn't read client certificate '%s': %s", cc_filename, error ? error->message : "(unknown)");
 	}
 
 	/* TLS CA certificate */
 	ca_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (self->ca_cert_button));
 
 	format = NM_SETTING_802_1X_CK_FORMAT_UNKNOWN;
-	if (parent->phase2) {
-		g_autoptr(GError) error = NULL;
-		if (!nm_setting_802_1x_set_phase2_ca_cert (s_8021x, ca_filename, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error)) {
-			g_warning ("Couldn't read phase2 CA certificate '%s': %s", ca_filename, error ? error->message : "(unknown)");
-			ca_cert_error = TRUE;
-		}
-	} else {
-		g_autoptr(GError) error = NULL;
-		if (!nm_setting_802_1x_set_ca_cert (s_8021x, ca_filename, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error)) {
-			g_warning ("Couldn't read CA certificate '%s': %s", ca_filename, error ? error->message : "(unknown)");
-			ca_cert_error = TRUE;
-		}
+	if (!nm_setting_802_1x_set_ca_cert (s_8021x, ca_filename, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error2)) {
+		g_warning ("Couldn't read CA certificate '%s': %s", ca_filename, error2 ? error2->message : "(unknown)");
+		ca_cert_error = TRUE;
 	}
-	eap_method_ca_cert_ignore_set (parent, connection, ca_filename, ca_cert_error);
+	eap_method_ca_cert_ignore_set (method, connection, ca_filename, ca_cert_error);
 }
 
 static void
-private_key_picker_helper (EAPMethod *parent, const char *filename, gboolean changed)
+private_key_picker_helper (EAPMethodTLS *self, const char *filename, gboolean changed)
 {
-	EAPMethodTLS *self = (EAPMethodTLS *) parent;
 	g_autoptr(NMSetting8021x) setting = NULL;
 	NMSetting8021xCKFormat cert_format = NM_SETTING_802_1X_CK_FORMAT_UNKNOWN;
 	const char *password;
@@ -263,7 +254,7 @@ private_key_picker_helper (EAPMethod *parent, const char *filename, gboolean cha
 		GtkWidget *toplevel;
 		GtkWindow *parent_window = NULL;
 
-		toplevel = gtk_widget_get_toplevel (parent->ui_widget);
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
 		if (gtk_widget_is_toplevel (toplevel))
 			parent_window = GTK_WINDOW (toplevel);
 
@@ -284,12 +275,12 @@ private_key_picker_helper (EAPMethod *parent, const char *filename, gboolean cha
 static void
 private_key_picker_file_set_cb (GtkWidget *chooser, gpointer user_data)
 {
-	EAPMethod *parent = (EAPMethod *) user_data;
+	EAPMethodTLS *self = user_data;
 	g_autofree gchar *filename = NULL;
 
 	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
 	if (filename)
-		private_key_picker_helper (parent, filename, TRUE);
+		private_key_picker_helper (self, filename, TRUE);
 }
 
 static void reset_filter (GtkWidget *widget, GParamSpec *spec, gpointer user_data)
@@ -307,14 +298,13 @@ typedef NMSetting8021xCKScheme (*SchemeFunc)  (NMSetting8021x *setting);
 static void
 changed_cb (EAPMethodTLS *self)
 {
-	wireless_security_notify_changed (self->sec_parent);
+	eap_method_emit_changed (EAP_METHOD (self));
 }
 
 static void
-setup_filepicker (GtkFileChooserButton *button,
+setup_filepicker (EAPMethodTLS *self,
+                  GtkFileChooserButton *button,
                   const char *title,
-                  WirelessSecurity *ws_parent,
-                  EAPMethod *parent,
                   NMSetting8021x *s_8021x,
                   SchemeFunc scheme_func,
                   PathFunc path_func,
@@ -341,12 +331,12 @@ setup_filepicker (GtkFileChooserButton *button,
 	if (privkey) {
 		g_signal_connect (button, "selection-changed",
 		                  (GCallback) private_key_picker_file_set_cb,
-		                  parent);
+		                  self);
 		if (filename)
-			private_key_picker_helper (parent, filename, FALSE);
+			private_key_picker_helper (self, filename, FALSE);
 	}
 
-	g_signal_connect_swapped (button, "selection-changed", G_CALLBACK (changed_cb), parent);
+	g_signal_connect_swapped (button, "selection-changed", G_CALLBACK (changed_cb), self);
 
 	filter = eap_method_default_file_chooser_filter_new (privkey);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (button), filter);
@@ -361,82 +351,140 @@ setup_filepicker (GtkFileChooserButton *button,
 }
 
 static void
-update_secrets (EAPMethod *parent, NMConnection *connection)
+update_secrets (EAPMethod *method, NMConnection *connection)
 {
-	EAPMethodTLS *self = (EAPMethodTLS *) parent;
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
 	NMSetting8021x *s_8021x;
-	HelperSecretFunc password_func;
-	SchemeFunc scheme_func;
-	PathFunc path_func;
 	const char *filename;
 
-	if (parent->phase2) {
-		password_func = (HelperSecretFunc) nm_setting_802_1x_get_phase2_private_key_password;
-		scheme_func = nm_setting_802_1x_get_phase2_private_key_scheme;
-		path_func = nm_setting_802_1x_get_phase2_private_key_path;
-	} else {
-		password_func = (HelperSecretFunc) nm_setting_802_1x_get_private_key_password;
-		scheme_func = nm_setting_802_1x_get_private_key_scheme;
-		path_func = nm_setting_802_1x_get_private_key_path;
-	}
-
 	helper_fill_secret_entry (connection,
 	                          self->private_key_password_entry,
 	                          NM_TYPE_SETTING_802_1X,
-	                          password_func);
+	                          (HelperSecretFunc) nm_setting_802_1x_get_private_key_password);
 
 	/* Set the private key filepicker button path if we have a private key */
 	s_8021x = nm_connection_get_setting_802_1x (connection);
-	if (s_8021x && (scheme_func (s_8021x) == NM_SETTING_802_1X_CK_SCHEME_PATH)) {
-		filename = path_func (s_8021x);
+	if (s_8021x && (nm_setting_802_1x_get_private_key_scheme (s_8021x) == NM_SETTING_802_1X_CK_SCHEME_PATH)) {
+		filename = nm_setting_802_1x_get_private_key_path (s_8021x);
 		if (filename)
 			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (self->private_key_button), filename);
 	}
 }
 
+static GtkWidget *
+get_default_field (EAPMethod *method)
+{
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
+	return GTK_WIDGET (self->identity_entry);
+}
+
+static const gchar *
+get_password_flags_name (EAPMethod *method)
+{
+	return NM_SETTING_802_1X_PRIVATE_KEY_PASSWORD;
+}
+
+static const gchar *
+get_username (EAPMethod *method)
+{
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
+	return self->username;
+}
+
+static void
+set_username (EAPMethod *method, const gchar *username)
+{
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
+	g_free (self->username);
+	self->username = g_strdup (username);
+}
+
+static const gchar *
+get_password (EAPMethod *method)
+{
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
+	return self->password;
+}
+
+static void
+set_password (EAPMethod *method, const gchar *password)
+{
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
+	g_free (self->password);
+	self->password = g_strdup (password);
+}
+
+static const gboolean
+get_show_password (EAPMethod *method)
+{
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
+	return self->show_password;
+}
+
+static void
+set_show_password (EAPMethod *method, gboolean show_password)
+{
+	EAPMethodTLS *self = EAP_METHOD_TLS (method);
+	self->show_password = show_password;
+}
+
+static void
+eap_method_tls_init (EAPMethodTLS *self)
+{
+	gtk_widget_init_template (GTK_WIDGET (self));
+	self->username = g_strdup ("");
+	self->password = g_strdup ("");
+}
+
+static void
+eap_method_tls_class_init (EAPMethodTLSClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+        GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+
+	object_class->dispose = eap_method_tls_dispose;
+
+	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/ControlCenter/network/eap-method-tls.ui");
+
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, ca_cert_button);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, ca_cert_label);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, ca_cert_not_required_check);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, identity_entry);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, identity_label);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, private_key_button);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, private_key_label);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, private_key_password_entry);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, private_key_password_label);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, show_password_check);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, user_cert_button);
+	gtk_widget_class_bind_template_child (widget_class, EAPMethodTLS, user_cert_label);
+}
+
+static void
+eap_method_iface_init (EAPMethodInterface *iface)
+{
+	iface->validate = validate;
+	iface->add_to_size_group = add_to_size_group;
+	iface->fill_connection = fill_connection;
+	iface->update_secrets = update_secrets;
+	iface->get_default_field = get_default_field;
+	iface->get_password_flags_name = get_password_flags_name;
+	iface->get_username = get_username;
+	iface->set_username = set_username;
+	iface->get_password = get_password;
+	iface->set_password = set_password;
+	iface->get_show_password = get_show_password;
+	iface->set_show_password = set_show_password;
+}
+
 EAPMethodTLS *
-eap_method_tls_new (WirelessSecurity *ws_parent,
-                    NMConnection *connection,
-                    gboolean phase2,
-                    gboolean secrets_only)
+eap_method_tls_new (NMConnection *connection)
 {
 	EAPMethodTLS *self;
-	EAPMethod *parent;
 	NMSetting8021x *s_8021x = NULL;
 	gboolean ca_not_required = FALSE;
 
-	parent = eap_method_init (sizeof (EAPMethodTLS),
-	                          validate,
-	                          add_to_size_group,
-	                          fill_connection,
-	                          update_secrets,
-	                          NULL,
-	                          "/org/gnome/ControlCenter/network/eap-method-tls.ui",
-	                          "grid",
-	                          "identity_entry",
-	                          phase2);
-	if (!parent)
-		return NULL;
-
-	parent->password_flags_name = phase2 ?
-	                                NM_SETTING_802_1X_PHASE2_PRIVATE_KEY_PASSWORD :
-	                                NM_SETTING_802_1X_PRIVATE_KEY_PASSWORD;
-	self = (EAPMethodTLS *) parent;
-	self->sec_parent = ws_parent;
-	self->editing_connection = secrets_only ? FALSE : TRUE;
-
-	self->ca_cert_button = GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (parent->builder, "ca_cert_button"));
-	self->ca_cert_label = GTK_LABEL (gtk_builder_get_object (parent->builder, "ca_cert_label"));
-	self->ca_cert_not_required_check = GTK_CHECK_BUTTON (gtk_builder_get_object (parent->builder, "ca_cert_not_required_check"));
-	self->identity_entry = GTK_ENTRY (gtk_builder_get_object (parent->builder, "identity_entry"));
-	self->identity_label = GTK_LABEL (gtk_builder_get_object (parent->builder, "identity_label"));
-	self->private_key_button = GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (parent->builder, "private_key_button"));
-	self->private_key_label = GTK_LABEL (gtk_builder_get_object (parent->builder, "private_key_label"));
-	self->private_key_password_entry = GTK_ENTRY (gtk_builder_get_object (parent->builder, "private_key_password_entry"));
-	self->private_key_password_label = GTK_LABEL (gtk_builder_get_object (parent->builder, "private_key_password_label"));
-	self->show_password_check = GTK_CHECK_BUTTON (gtk_builder_get_object (parent->builder, "show_password_check"));
-	self->user_cert_button = GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (parent->builder, "user_cert_button"));
-	self->user_cert_label = GTK_LABEL (gtk_builder_get_object (parent->builder, "user_cert_label"));
+	self = g_object_new (eap_method_tls_get_type (), NULL);
 
 	if (connection)
 		s_8021x = nm_connection_get_setting_802_1x (connection);
@@ -447,52 +495,44 @@ eap_method_tls_new (WirelessSecurity *ws_parent,
 	if (s_8021x && nm_setting_802_1x_get_identity (s_8021x))
 		gtk_entry_set_text (self->identity_entry, nm_setting_802_1x_get_identity (s_8021x));
 
-	setup_filepicker (self->user_cert_button,
+	setup_filepicker (self,
+	                  self->user_cert_button,
 	                  _("Choose your personal certificate"),
-	                  ws_parent, parent, s_8021x,
-	                  phase2 ? nm_setting_802_1x_get_phase2_client_cert_scheme : nm_setting_802_1x_get_client_cert_scheme,
-	                  phase2 ? nm_setting_802_1x_get_phase2_client_cert_path : nm_setting_802_1x_get_client_cert_path,
+	                  s_8021x,
+	                  nm_setting_802_1x_get_client_cert_scheme,
+	                  nm_setting_802_1x_get_client_cert_path,
 	                  FALSE, TRUE);
-	setup_filepicker (self->ca_cert_button,
+	setup_filepicker (self,
+	                  self->ca_cert_button,
 	                  _("Choose a Certificate Authority certificate"),
-	                  ws_parent, parent, s_8021x,
-	                  phase2 ? nm_setting_802_1x_get_phase2_ca_cert_scheme : nm_setting_802_1x_get_ca_cert_scheme,
-	                  phase2 ? nm_setting_802_1x_get_phase2_ca_cert_path : nm_setting_802_1x_get_ca_cert_path,
+	                  s_8021x,
+	                  nm_setting_802_1x_get_ca_cert_scheme,
+	                  nm_setting_802_1x_get_ca_cert_path,
 	                  FALSE, FALSE);
-	setup_filepicker (self->private_key_button,
+	setup_filepicker (self,
+	                  self->private_key_button,
 	                  _("Choose your private key"),
-	                  ws_parent, parent, s_8021x,
-	                  phase2 ? nm_setting_802_1x_get_phase2_private_key_scheme : nm_setting_802_1x_get_private_key_scheme,
-	                  phase2 ? nm_setting_802_1x_get_phase2_private_key_path : nm_setting_802_1x_get_private_key_path,
+	                  s_8021x,
+	                  nm_setting_802_1x_get_private_key_scheme,
+	                  nm_setting_802_1x_get_private_key_path,
 	                  TRUE, FALSE);
 
-	if (connection && eap_method_ca_cert_ignore_get (parent, connection))
+	if (connection && eap_method_ca_cert_ignore_get (EAP_METHOD (self), connection))
 		ca_not_required = !gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (self->ca_cert_button));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->ca_cert_not_required_check), ca_not_required);
 
 	/* Fill secrets, if any */
 	if (connection)
-		update_secrets (parent, connection);
+		update_secrets (EAP_METHOD (self), connection);
 
 	g_signal_connect_swapped (self->private_key_password_entry, "changed", G_CALLBACK (changed_cb), self);
 
 	/* Create password-storage popup menu for password entry under entry's secondary icon */
-	nma_utils_setup_password_storage (GTK_WIDGET (self->private_key_password_entry), 0, (NMSetting *) s_8021x, parent->password_flags_name,
-	                                  FALSE, secrets_only);
+	nma_utils_setup_password_storage (GTK_WIDGET (self->private_key_password_entry), 0, (NMSetting *) s_8021x, NM_SETTING_802_1X_PRIVATE_KEY_PASSWORD,
+	                                  FALSE, FALSE);
 
 	g_signal_connect_swapped (self->show_password_check, "toggled", G_CALLBACK (show_toggled_cb), self);
 
-	if (secrets_only) {
-		gtk_widget_set_sensitive (GTK_WIDGET (self->identity_entry), FALSE);
-		gtk_widget_hide (GTK_WIDGET (self->user_cert_label));
-		gtk_widget_hide (GTK_WIDGET (self->user_cert_button));
-		gtk_widget_hide (GTK_WIDGET (self->private_key_label));
-		gtk_widget_hide (GTK_WIDGET (self->private_key_button));
-		gtk_widget_hide (GTK_WIDGET (self->ca_cert_label));
-		gtk_widget_hide (GTK_WIDGET (self->ca_cert_button));
-		gtk_widget_hide (GTK_WIDGET (self->ca_cert_not_required_check));
-	}
-
 	return self;
 }
 
