@@ -17,19 +17,21 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2010 Red Hat, Inc.
+ * Copyright 2007 - 2014 Red Hat, Inc.
  */
 
-#ifndef WS_WPA_EAP_H
-#define WS_WPA_EAP_H
+#pragma once
 
-#include <nm-connection.h>
+#include <gtk/gtk.h>
+#include <NetworkManager.h>
 
-typedef struct _WirelessSecurityWPAEAP WirelessSecurityWPAEAP;
+G_BEGIN_DECLS
 
-WirelessSecurityWPAEAP * ws_wpa_eap_new (NMConnection *connection,
-                                         gboolean is_editor,
-                                         gboolean secrets_only);
+G_DECLARE_FINAL_TYPE (WirelessSecurityWPAEAP, ws_wpa_eap, WS, WPA_EAP, GtkGrid)
 
-#endif /* WS_WPA_EAP_H */
+WirelessSecurityWPAEAP *ws_wpa_eap_new             (NMConnection *connection);
 
+void                    ws_wpa_eap_fill_connection (WirelessSecurityWPAEAP *sec,
+                                                    NMConnection *connection);
+
+G_END_DECLS
