@@ -20,6 +20,7 @@
 
 #include <gtk/gtk.h>
 #include <pulse/pulseaudio.h>
+#include <gvc-mixer-control.h>
 #include <gvc-mixer-stream.h>
 
 #include "cc-sound-enums.h"
@@ -29,11 +30,14 @@ G_BEGIN_DECLS
 #define CC_TYPE_VOLUME_SLIDER (cc_volume_slider_get_type ())
 G_DECLARE_FINAL_TYPE (CcVolumeSlider, cc_volume_slider, CC, VOLUME_SLIDER, GtkBox)
 
-void cc_volume_slider_set_stream       (CcVolumeSlider *slider,
-                                        GvcMixerStream *stream,
-                                        CcStreamType    type);
+void cc_volume_slider_set_mixer_control (CcVolumeSlider  *slider,
+                                         GvcMixerControl *mixer_control);
 
-void cc_volume_slider_set_is_amplified (CcVolumeSlider *slider,
-                                        gboolean        is_amplified);
+void cc_volume_slider_set_stream        (CcVolumeSlider  *slider,
+                                         GvcMixerStream  *stream,
+                                         CcStreamType     type);
+
+void cc_volume_slider_set_is_amplified  (CcVolumeSlider  *slider,
+                                         gboolean         is_amplified);
 
 G_END_DECLS
