@@ -4,9 +4,10 @@
  * SPDX-License-Identifier: LGPL-2.1+
  */
 
+#include "config.h"
 #include "hdy-action-row.h"
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 /**
  * SECTION:hdy-action-row
@@ -51,7 +52,7 @@ typedef struct
 
 static void hdy_action_row_buildable_init (GtkBuildableIface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (HdyActionRow, hdy_action_row, GTK_TYPE_LIST_BOX_ROW,
+G_DEFINE_TYPE_WITH_CODE (HdyActionRow, hdy_action_row, HDY_TYPE_PREFERENCES_ROW,
                          G_ADD_PRIVATE (HdyActionRow)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
                          hdy_action_row_buildable_init))
@@ -71,10 +72,11 @@ enum {
 static GParamSpec *props[LAST_PROP];
 
 static void
-row_activated_cb (HdyActionRow        *self,
+row_activated_cb (HdyActionRow  *self,
                   GtkListBoxRow *row)
 {
-  if (self == HDY_ACTION_ROW (row))
+  /* No need to use GTK_LIST_BOX_ROW() for a pointer comparison. */
+  if ((GtkListBoxRow *) self == row)
     hdy_action_row_activate (self);
 }
 
@@ -379,7 +381,7 @@ hdy_action_row_class_init (HdyActionRowClass *klass)
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/handy/dialer/ui/hdy-action-row.ui");
+                                               "/sm/puri/handy/ui/hdy-action-row.ui");
   gtk_widget_class_bind_template_child_private (widget_class, HdyActionRow, box);
   gtk_widget_class_bind_template_child_private (widget_class, HdyActionRow, header);
   gtk_widget_class_bind_template_child_private (widget_class, HdyActionRow, image);
@@ -476,6 +478,7 @@ hdy_action_row_set_title (HdyActionRow *self,
   g_return_if_fail (HDY_IS_ACTION_ROW (self));
 
   priv = hdy_action_row_get_instance_private (self);
+  hdy_preferences_row_set_title (HDY_PREFERENCES_ROW (self), title);
 
   if (g_strcmp0 (gtk_label_get_text (priv->title), title) == 0)
     return;
@@ -722,6 +725,7 @@ hdy_action_row_set_use_underline (HdyActionRow *self,
     return;
 
   priv->use_underline = !!use_underline;
+  hdy_preferences_row_set_use_underline (HDY_PREFERENCES_ROW (self), priv->use_underline);
   gtk_label_set_use_underline (priv->title, priv->use_underline);
   gtk_label_set_use_underline (priv->subtitle, priv->use_underline);
   gtk_label_set_mnemonic_widget (priv->title, GTK_WIDGET (self));
