@@ -4,7 +4,8 @@
  * SPDX-License-Identifier: LGPL-2.1+
  */
 
-#include <glib/gi18n.h>
+#include "config.h"
+#include <glib/gi18n-lib.h>
 
 #include "hdy-dialer.h"
 #include "hdy-dialer-button.h"
@@ -365,7 +366,7 @@ hdy_dialer_constructed (GObject *object)
                            self,
                            G_CONNECT_SWAPPED);
 
-  /* In GTK+4 we can just use the icon-name property */
+  /* In GTK 4 we can just use the icon-name property */
   image = gtk_image_new_from_icon_name ("edit-clear-symbolic",
                                         GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (priv->btn_del, image);
@@ -476,7 +477,7 @@ hdy_dialer_class_init (HdyDialerClass *klass)
     g_signal_new ("deleted",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (HdyDialerClass, submitted),
+                  0,
                   NULL, NULL, NULL,
                   G_TYPE_NONE,
                   0);
@@ -503,7 +504,7 @@ hdy_dialer_class_init (HdyDialerClass *klass)
                   G_TYPE_CHAR);
 
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/handy/dialer/ui/hdy-dialer.ui");
+                                               "/sm/puri/handy/ui/hdy-dialer.ui");
   for (int i=0; i < 10; i++) {
     g_autofree gchar *name = g_strdup_printf("btn_%d", i);
     g_return_if_fail (name);
@@ -518,6 +519,7 @@ hdy_dialer_class_init (HdyDialerClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, HdyDialer, btn_submit);
   gtk_widget_class_bind_template_child_private (widget_class, HdyDialer, btn_del);
 
+  gtk_widget_class_set_accessible_role (widget_class, ATK_ROLE_DIAL);
   gtk_widget_class_set_css_name (widget_class, "hdydialer");
 }
 
@@ -546,6 +548,7 @@ hdy_dialer_init (HdyDialer *self)
 
   priv->number = g_string_new (NULL);
   priv->cycle_btn = NULL;
+  priv->show_action_buttons = TRUE;
 }
 
 /**
@@ -648,6 +651,10 @@ hdy_dialer_set_show_action_buttons (HdyDialer *self,
   g_return_if_fail (HDY_IS_DIALER (self));
 
   priv = hdy_dialer_get_instance_private (self);
+
+  if (priv->show_action_buttons == show)
+    return;
+
   priv->show_action_buttons = show;
 
   gtk_widget_set_visible (GTK_WIDGET (priv->btn_submit), show);
